@@ -4,61 +4,74 @@
 from pathlib import Path
 import numpy as np
 import pandas as pd
-import datetime
-from covsirphy.util.error import SubsetNotFoundError
+from datetime import datetime
+from covsirphy.util.error import deprecate, SubsetNotFoundError
 from covsirphy.cleaning.cbase import CleaningBase
 
 
 class VaccineData(CleaningBase):
     """
-    Dataset regarding vaccination retrieved from "Our World In Data".
-    https://github.com/owid/covid-19-data/tree/master/public/data
-    https://ourworldindata.org/coronavirus
+    Data cleaning of vaccination dataset.
 
     Args:
-        filename (str or pathlib.path): CSV filename to save the raw dataset
-        force (bool): if True, always download the dataset from the server
-        verbose (int): level of verbosity
+        filename (str or None): CSV filename of the dataset
+        data (pandas.DataFrame or None):
+            Index
+                reset index
+            Columns
+                - Date: Observation date
+                - Country: country/region name
+                - ISO3: ISO 3166-1 alpha-3, like JPN
+                - Province: province/prefecture/state name
+                - Product: vaccine product names
+                - Vaccinations: cumulative number of vaccinations
+                - Vaccinated_once: cumulative number of people who received at least one vaccine dose
+                - Vaccinated_full: cumulative number of people who received all doses prescrived by the protocol
+        citation (str or None): citation or None (empty)
+        kwargs: the other arguments will be ignored
+
+    Note:
+        Either @filename (high priority) or @data must be specified.
 
     Note:
         Columns of VaccineData.cleaned():
             - Date (pandas.TimeStamp): observation dates
             - Country (pandas.Category): country (or province) names
             - ISO3 (pandas.Category): ISO3 codes
-            - Product (pandas.Category): product names
+            - Province (pandas.Category): province/prefecture/state name
+            - Product (pandas.Category): vaccine product names
             - Vaccinations (int): cumulative number of vaccinations
             - Vaccinated_once (int): cumulative number of people who received at least one vaccine dose
             - Vaccinated_full (int): cumulative number of people who received all doses prescrived by the protocol
     """
-    # URL
-    URL = "https://raw.githubusercontent.com/owid/covid-19-data/master/public/data/vaccinations/"
-    URL_REC = f"{URL}vaccinations.csv"
-    URL_LOC = f"{URL}locations.csv"
-    # Columns
-    VAC_COLS = [
-        CleaningBase.DATE, CleaningBase.COUNTRY, CleaningBase.ISO3, CleaningBase.PRODUCT,
-        CleaningBase.VAC, CleaningBase.V_ONCE, CleaningBase.V_FULL]
-    VAC_SUBSET_COLS = [CleaningBase.DATE, CleaningBase.VAC, CleaningBase.V_ONCE, CleaningBase.V_FULL]
 
-    def __init__(self, filename, force=False, verbose=1):
-        Path(filename).parent.mkdir(exist_ok=True, parents=True)
-        if Path(filename).exists() and not force:
-            try:
-                self._raw = self.load(filename)
-            except KeyError:
-                # Error when the local dataset does not have necessary columns
-                # Raised when new CovsirPhy version requires additional columns
-                self._raw = self._retrieve(filename=filename, verbose=verbose)
-        else:
-            self._raw = self._retrieve(filename=filename, verbose=verbose)
-        self._cleaned_df = self._cleaning()
-        self._citation = "Hasell, J., Mathieu, E., Beltekian, D. et al." \
-            " A cross-country database of COVID-19 testing. Sci Data 7, 345 (2020)." \
-            " https://doi.org/10.1038/s41597-020-00688-8"
+    def __init__(self, filename=None, data=None, citation=None, **kwargs):
+        self._subset_cols = [self.DATE, self.VAC, self.V_ONCE, self.V_FULL]
+        self._raw_cols = [
+            self.DATE, self.ISO3, self.COUNTRY, self.PROVINCE, self.PRODUCT,
+            self.VAC, self.V_ONCE, self.V_FULL]
+        # Raw data
+        self._raw = self._parse_raw(filename, data, self._raw_cols)
+        # Backward compatibility
+        if self._raw.empty:
+            self._raw = self._retrieve(filename, **kwargs)
+        if self.PROVINCE not in self._raw:
+            self._raw[self.PROVINCE] = self.UNKNOWN
+        # Data cleaning
+        self._cleaned_df = pd.DataFrame(columns=self._raw_cols) if self._raw.empty else self._cleaning()
+        # Citation
+        self._citation = citation or ""
         # Directory that save the file
-        self._dirpath = Path(filename or "input").resolve().parent
+        if filename is None:
+            self._dirpath = Path("input")
+        else:
+            Path(filename).parent.mkdir(exist_ok=True, parents=True)
+            self._dirpath = Path(filename).resolve().parent
 
-    def _retrieve(self, filename, verbose=1):
+    @deprecate(
+        "vaccine_data = cs.VaccineData()",
+        new="vaccine_data = cs.DataLoader().vaccine()", version="2.21.0-iota-fu1")
+    def _retrieve(self, filename, verbose=1, **kwargs):
         """
         Retrieve the dataset from server.
         Args:
@@ -69,6 +82,9 @@ class VaccineData(CleaningBase):
                 Index reset index
                 Columns Date, Country, Product, Vaccinations
         """
+        URL = "https://raw.githubusercontent.com/owid/covid-19-data/master/public/data/vaccinations/"
+        URL_REC = f"{URL}vaccinations.csv"
+        URL_LOC = f"{URL}locations.csv"
         # Show URL
         if verbose:
             print("Retrieving COVID-19 vaccination dataset from https://github.com/owid/covid-19-data/")
@@ -79,8 +95,8 @@ class VaccineData(CleaningBase):
             "people_vaccinated": self.V_ONCE,
             "people_fully_vaccinated": self.V_FULL,
         }
-        rec_df = self.load(self.URL_REC, columns=list(set(rename_dict) - set(["vaccines"])))
-        loc_df = self.load(self.URL_LOC, columns=["location", "vaccines"])
+        rec_df = pd.read_csv(URL_REC, usecols=list(set(rename_dict) - set(["vaccines"])))
+        loc_df = pd.read_csv(URL_LOC, usecols=["location", "vaccines"])
         df = rec_df.merge(loc_df, how="left", on="location")
         df = df.rename(rename_dict, axis=1)
         # Save the dataframe as CSV file
@@ -99,7 +115,8 @@ class VaccineData(CleaningBase):
                 - Date (pandas.TimeStamp): observation dates
                 - Country (pandas.Category): country (or province) names
                 - ISO3 (pandas.Category): ISO3 codes
-                - Product (pandas.Category): product names
+                - Province: province/prefecture/state name
+                - Product (pandas.Category): vaccine product names
                 - Vaccinations (int): cumulative number of vaccinations
                 - Vaccinated_once (int): cumulative number of people who received at least one vaccine dose
                 - Vaccinated_full (int): cumulative number of people who received all doses prescrived by the protocol
@@ -107,37 +124,33 @@ class VaccineData(CleaningBase):
         df = self._raw.copy()
         # Date
         df[self.DATE] = pd.to_datetime(df[self.DATE])
-        for col in [self.COUNTRY, self.ISO3, self.PRODUCT]:
-            df[col] = df[col].astype("category")
         # Fill in NA values
         for col in [self.VAC, self.V_ONCE, self.V_FULL]:
             df[col] = pd.to_numeric(df[col], errors="coerce")
-            df[col] = df.groupby(self.ISO3)[col].fillna(method="ffill").fillna(0).astype(np.int64)
-        today_date = datetime.datetime.today().replace(hour=00, minute=00, second=00, microsecond=00)
-        for country in df.Country.unique():
-            subset_df = df.loc[df[self.COUNTRY] == country]
-            # Add any missing dates up until today
-            if subset_df[self.DATE].max() < today_date:
-                new_dates = pd.date_range(
-                    subset_df[self.DATE].max() + datetime.timedelta(days=1), today_date)
-                subset_df = subset_df.reset_index(drop=True)
-                keep_index = subset_df[self.VAC].idxmax() + 1
-                new_df = pd.DataFrame(index=new_dates, columns=subset_df.columns)
-                new_df.index.name = self.DATE
-                new_df = new_df.drop(self.DATE, axis=1).reset_index()
-                subset_df = pd.concat([subset_df, new_df], axis=0, ignore_index=True).ffill()
-                subset_df = subset_df.loc[keep_index:]
-                df = pd.concat([df, subset_df], axis=0, ignore_index=True)
-        df.sort_values(by=[self.COUNTRY, self.DATE], ignore_index=True, inplace=True)
-        return df.loc[:, self.VAC_COLS]
+        country_df = df.loc[:, [self.COUNTRY, self.ISO3, self.PRODUCT]].drop_duplicates()
+        # Extent dates to today
+        today_date = datetime.today().replace(hour=00, minute=00, second=00, microsecond=00)
+        df = df.pivot_table(
+            values=[self.VAC, self.V_ONCE, self.V_FULL],
+            index=self.DATE, columns=[self.COUNTRY, self.PROVINCE], aggfunc="last")
+        df = df.reindex(pd.date_range(df.index[0], today_date, freq="D"))
+        df.index.name = self.DATE
+        df = df.ffill().fillna(0).astype(np.int64).stack().stack().reset_index()
+        df.sort_values(by=[self.COUNTRY, self.PROVINCE, self.DATE], ignore_index=True, inplace=True)
+        df = df.merge(country_df, on=self.COUNTRY)
+        # Set dtype for category data
+        for col in [self.COUNTRY, self.ISO3, self.PROVINCE, self.PRODUCT]:
+            df[col] = df[col].astype("category")
+        return df.loc[:, self._raw_cols]
 
-    def subset(self, country, product=None, start_date=None, end_date=None):
+    def subset(self, country, province=None, product=None, start_date=None, end_date=None):
         """
         Return subset of the country/province and start/end date.
 
         Args:
-            country (str or None): country name or ISO3 code
-            product (str or None): product name
+            country (str): country name or ISO3 code
+            province (str or None): province name
+            product (str or None): vaccine product name
             start_date (str or None): start date, like 22Jan2020
             end_date (str or None): end date, like 01Feb2020
 
@@ -146,7 +159,7 @@ class VaccineData(CleaningBase):
                 Index
                     reset index
                 Columns
-                    - Date (pandas.TimeStamp): observation date
+                    - Date (pandas.Timestamp): observation date
                     - Vaccinations (int): the number of vaccinations
                     - Vaccinated_once (int): cumulative number of people who received at least one vaccine dose
                     - Vaccinated_full (int): cumulative number of people who received all doses prescrived by the protocol
@@ -154,7 +167,7 @@ class VaccineData(CleaningBase):
         df = self._cleaned_df.copy()
         # Subset by country
         country_alias = self.ensure_country_name(country)
-        df = df.loc[df[self.COUNTRY] == country_alias]
+        df = df.loc[(df[self.COUNTRY] == country_alias) & (df[self.PROVINCE] == (province or self.UNKNOWN))]
         # Subset by product name
         if product is not None:
             df = df.loc[df[self.PRODUCT] == product]
@@ -169,7 +182,7 @@ class VaccineData(CleaningBase):
             raise SubsetNotFoundError(
                 country=country, country_alias=country_alias, province=product,
                 start_date=start_date, end_date=end_date)
-        return df.loc[:, self.VAC_SUBSET_COLS].reset_index(drop=True)
+        return df.loc[:, self._subset_cols].reset_index(drop=True)
 
     def records(self, country, product=None, start_date=None, end_date=None):
         """
@@ -213,7 +226,7 @@ class VaccineData(CleaningBase):
         df = df.loc[df[self.COUNTRY] == "World"]
         # Resampling
         df = df.set_index(self.DATE).resample("D").sum()
-        return df.reset_index()
+        return df.reset_index()[self._subset_cols]
 
     def map(self, country=None, variable="Vaccinations", date=None, **kwargs):
         """
